/**
 * Netwosix Web Admin - agentd
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#include "agentd.h"



/**
 */
void syntax(void);


/**
 */
void supported_connection(void);


#ifdef AL_SSL
static const char * supported[] = { "stdio", "tcp", "ssl", NULL };
#else
static const char * supported[] = { "stdio", "tcp", NULL };
#endif


/**
 */
int main(int argc, char ** argv)
{
	AlConnectionManager     * connection_manager = NULL;
	AlConnection            * connection = NULL;
	AgentdRequestPacket     * request_packet = NULL;
	AgentdResponsePacket    * response_packet = NULL;
	pid_t                     pid = 0;
	char                    * connection_type = NULL;
	char                    * s_port;
	int                       port;
	AlXmlNode               * node;
	AlString                * s;
	int                       i;



	/** Parse the command line */
	for (i=1;i<argc;i++)
	{

		if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help"))
		{
			syntax();
		}
		else if (!strcmp(argv[i],"-v") || !strcmp(argv[i],"--version"))
		{
			al_exit(0, "%s\n", AGENTD_VERSION);
		}
		else if (!strcmp(argv[i],"-c") || !strcmp(argv[i],"--connection"))
		{
			if ((++i) < argc)
			{
				connection_type = argv[i];
				if (!strcmp(connection_type,"list"))
					supported_connection();
			}
			else
			{
				syntax();
			}
		}
		else
		{
			syntax();
		}

	}



	if ((node = al_xml_get_node(agentd_config(), "connection")) == NULL)
		al_exit(1, "%s\nInvalid config file.\n",AGENTD_VERSION);

	if (!(connection_manager = al_connection_manager_from_string(connection_type, supported, node)))
		al_exit(1, "%s\nInvalid connection type.\n", AGENTD_VERSION);
		

	openlog("agentd", LOG_PID, LOG_DAEMON);

	while (1)
	{
		connection = al_connection_wait(connection_manager);
		if (connection == NULL) break;

		if (al_connection_need_fork(connection_manager)) pid = fork();
		if (pid == 0 || !al_connection_need_fork(connection_manager))
		{
			while (1)
			{
				s = al_connection_read(connection);
				if (s == NULL) break;

				request_packet = agentd_request_packet_from_xml_string(s);
				al_string_delete(s);

				response_packet = agentd_response_packet_from_request_packet(request_packet);
				if (response_packet)
				{
					s = agentd_response_packet_to_xml_string(response_packet);
					al_connection_send(connection, s);
					al_string_delete(s);
				}
			}
			if (al_connection_need_fork(connection_manager)) break;
		}

	}


	if (connection_manager) al_connection_manager_delete(connection_manager);


	al_exit(0, NULL);

	return 0;
}


/**
 */
void syntax(void)
{
	printf("%s\n", AGENTD_VERSION);

	printf("Usage: agentd [OPTION ...]\n");
	printf("\n");
	printf(" Options:\n");
	printf("\n");
	printf("  -c, --connection       Override the connection type given in config file.\n");
	printf("                         Use -c list to show the supported connection type.\n");
	printf("  -h, --help             Give this help list.\n");
	printf("  -v, --version          Show version and exit.\n");

	al_exit(0, NULL);
}


/**
 */
void supported_connection(void)
{
	int i;
	
	printf("%s\n", AGENTD_VERSION);
	printf(" Supported connection:\n\n");
	printf("\n");
	
	for(i=0;supported[i];i++)
		printf("  %s\n", supported[i]);

	al_exit(0, NULL);
}
