/**
 * Netwosix Web Admin - agents
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#include "agents.h"



/**
 */
void syntax(void);


/**
 */
void supported_connection(void);


#ifdef AL_SSL
static const char * supported[] = { "stdio", "tcp", "ssl", NULL };
#else
static const char * supported[] = { "stdio", "tcp", NULL };
#endif


/**
 */
int main(int argc, char ** argv)
{
	AlConnectionManager     * connection_manager = NULL;
	AlConnection            * connection = NULL;
	char                    * connection_type = NULL;
	char                    * s_port;
	int                       port;
	AlXmlNode               * node;
	AlString                * s;
	int                       i;
	const char              * server = NULL;



	/** Parse the command line */
	for (i=1;i<argc;i++)
	{

		if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help"))
		{
			syntax();
		}
		else if (!strcmp(argv[i],"-v") || !strcmp(argv[i],"--version"))
		{
			al_exit(0, "%s\n", AGENTS_VERSION);
		}
		else if (!strcmp(argv[i],"-c") || !strcmp(argv[i],"--connection"))
		{
			if ((++i) < argc)
			{
				connection_type = argv[i];
				if (!strcmp(connection_type,"list"))
					supported_connection();
			}
			else
			{
				syntax();
			}
		}
		else if (!strcmp(argv[i],"-s") || !strcmp(argv[i],"--server"))
		{
			if ((++i) < argc)
			{
				server = argv[i];
			}
			else
			{
				syntax();
			}
		}
		else
		{
			syntax();
		}

	}


	if ((node = al_xml_get_node(agents_config(), "connection")) == NULL)
		al_exit(1, "%s\nInvalid config file.\n", AGENTS_VERSION);

	if (!(connection_manager = al_connection_manager_from_string(connection_type, supported, node)))
		al_exit(1, "%s\nInvalid connection type.\n", AGENTS_VERSION);

	if (server == NULL)
		syntax();

	openlog("agents", LOG_PID, LOG_DAEMON);

	if ((connection = al_connection_connect(connection_manager, server)) != NULL)
	{

		s = al_read_file(stdin);
		if (s)
		{
			if (!al_xml_verify_from_string(al_string_get(s)))
			{
				al_connection_send(connection, s);
				al_string_delete(s);
				s = al_connection_read(connection);
				if (s)
				{
					if (al_xml_verify_from_string(al_string_get(s)))
						syslog(LOG_ERR, "Invalid xml packet read from %s.\n", server);
					else
						fprintf(stdout,"%s",al_string_get(s));
				}
				al_string_delete(s);
				al_connection_close(connection);

			}
			else
			{
				syslog(LOG_ERR, "Invalid xml packet from stdin.\n");
			}
		}
		if (connection_manager) al_connection_manager_delete(connection_manager);
	}
	else
	{
		al_exit(1, "%s\nCould not connect to %s\n", AGENTS_VERSION, server);
	}


	al_exit(0, NULL);

	return 0;
}


/**
 */
void syntax(void)
{

	printf("Usage: agents [OPTION ...] -s server\n");
	printf("\n");
	printf("  -s, --server           server to connect\n");
	printf(" Options:\n");
	printf("\n");
	printf("  -c, --connection       Override the connection type given in config file.\n");
	printf("                         Use -c list to show the supported connection type.\n");
	printf("  -h, --help             Give this help list.\n");
	printf("  -v, --version          Show version and exit.\n");


	al_exit(0, NULL);
}


/**
 */
void supported_connection(void)
{
	int i;

	printf("%s\n", AGENTS_VERSION);
	printf(" Supported connection:\n");
	printf("\n");
	for(i=0;supported[i];i++)
		printf("  %s\n", supported[i]);

	al_exit(0, NULL);
}
