/**
 * Netwosix Web Admin - libal
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */


#ifndef AL_SOCKET_H_
#define AL_SOCKET_H_


/**
 * Create a new connection manager based on socket.
 *
 * @param port listen port
 *
 * @return The new connection manager. Never return NULL.
 */
AlConnectionManager * al_socket_connection_manager_new(int port);


/**
 * Delete the connection manager.
 *
 * @param connection_manager connection manager to delete
 */
void al_socket_connection_manager_delete(AlConnectionManager * connection_manager);


/**
 * Wait for a new connection.
 *
 * @param connection_manager connection manager
 *
 * @return the new connection or NULL if not found.
 */
AlConnection * al_socket_connection_wait(AlConnectionManager * connection_manager);


/**
 * Do nothing.
 *
 * @param connection connection to close
 */
void al_socket_connection_close(AlConnection * connection);


/**
 * Read a request.
 *
 * @param connection_manager connection manager
 *
 * @return the new packet or NULL if not found.
 */
AlString * al_socket_connection_read(AlConnection * connection);


/**
 * Send a response.
 *
 * @param connection_manager connection manager
 * @param packet packet to send
 */
void al_socket_connection_send(AlConnection * connection, const AlString * response);


/**
 * Connect to a server.
 *
 * @param connection_manager the connection manager
 * @param server server to connect
 *
 * @return the new connection or NULL if fail
 */
AlConnection * al_socket_connection_connect(AlConnectionManager * connection_manager, const char * server);


#endif /* AL_SOCKET_H_ */
