/**
 * Netwosix Web Admin - libal
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */


#ifndef AL_STDIO_H_
#define AL_STDIO_H_


/**
 * Create a new connection manager based on stdio.
 *
 * @return The new connection manager. Never return NULL.
 */
AlConnectionManager * al_stdio_connection_manager_new(void);


/**
 * Delete the connection manager.
 *
 * @param connection_manager connection manager to delete
 */
void al_stdio_connection_manager_delete(AlConnectionManager * connection_manager);


/**
 * Wait for a new connection. There is a new connection when a EOF is find on stdio.
 *
 * @param connection_manager connection manager
 *
 * @return the new connection or NULL if not found.
 */
AlConnection * al_stdio_connection_wait(AlConnectionManager * connection_manager);


/**
 * Do nothing.
 *
 * @param connection connection to close
 */
void al_stdio_connection_close(AlConnection * connection);


/**
 * Read a request from stdio.
 *
 * @param connection_manager connection manager
 *
 * @return the read paket NULL if not found.
 */
AlString * al_stdio_connection_read(AlConnection * connection);


/**
 * Send a packet to stdio.
 *
 * @param connection_manager connection manager
 * @param packet packet to send
 */
void al_stdio_connection_send(AlConnection * connection, const AlString * packet);


/**
 * Do nothing
 */
AlConnection * al_stdio_connection_connect(AlConnectionManager * connection_manager, const char * server);



#endif /* AL_STDIO_H_ */
