/**
 * Netwosix Web Admin - libal
 * Copyright (C) 2006 netwosix team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * If you want to contact e-mail at info@netwosix.org
 */

#include "al.h"



/**
 */
AlConnectionManager * al_connection_manager_from_string(const char * connection_type, const char * supported[], AlXmlNode * node)
{
	AlConnectionManager * ret = NULL;
	int                   i;
	int                   port;
	char                * type = NULL;
	char                * s_port = NULL;


	type = al_strdup(connection_type);

	if (supported && type)
	{
		for (i=0;supported[i];i++)
		{
			if (!strcmp(type, supported[i]))
				break;
		}

		if (supported[i] == NULL)
			return ret;
	}

	if (type == NULL && node)
		type = al_xml_get_attribute_value(node,"type");

	if (type == NULL)
		return ret;

	if (!strcmp(type,"stdio"))
	{
		if ((ret = al_stdio_connection_manager_new()) == NULL)
			al_log (("Could not open %s connection.\n", type));
	}
	else if (!strcmp(type,"tcp"))
	{
		if (!(s_port = al_xml_get_attribute_value(node,"port")))
			al_log (("Invalid tcp port %d.\n", port));
		else
		{
			port = atoi(s_port);

			if (port <= 0 || port >= 0xFFFF)
				al_log(("Invalid tcp port %d.\n", port));
			else
				if ((ret = al_socket_connection_manager_new(port)) == NULL)
					al_log (("Could not open %s connection.\n", type));
		}
	}
#ifdef AL_SSL
	else if (!strcmp(type,"ssl"))
	{
		char * keyfile;
		char * dh_file;
		char * password;
		char * ca_list;

		
		if (!(keyfile = al_xml_get_attribute_value(node,"keyfile")))
			al_log(("No keyfile for SSL.\n"));
		
		if (!(dh_file = al_xml_get_attribute_value(node,"dh_file")))
			al_log(("No dh file for SSL.\n"));

		if (!(password = al_xml_get_attribute_value(node,"password")))
			al_log(("No password file for SSL.\n"));

		if (!(ca_list = al_xml_get_attribute_value(node,"ca_list")))
			al_log(("No ca_list file for SSL.\n"));

		if (!(s_port = al_xml_get_attribute_value(node,"port")))
		{
			al_log (("Invalid tcp port %d.\n", port));
		}
		else
		{
			port = atoi(s_port);

			if (port <= 0 || port >= 0xFFFF)
				al_log (("Invalid tcp port %d.\n", port));
			else
				if ((ret = al_ssl_connection_manager_new(port, keyfile, dh_file, ca_list, password)) == NULL)
					al_log (("Could not open %s connection.\n", type));
		}
	}
#endif
	else
	{
		al_log (("No suitable connection found.\n"));
	}


	return ret;
}
